/*
 * [The "BSD licence"]
 * Copyright (c) 2013 Dandelion
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Dandelion nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.github.dandelion.webanalytics.thymeleaf.processor;

import com.github.dandelion.core.asset.web.AssetsRequestContext;
import com.github.dandelion.thymeleaf.processor.DandelionAttrProcessor;
import com.github.dandelion.thymeleaf.util.ArgumentsUtil;
import com.github.dandelion.thymeleaf.util.AssetsFinalizerProcessorUtil;
import com.github.dandelion.thymeleaf.util.AttributesUtil;
import com.github.dandelion.webanalytics.core.WebAnalyticsRequestContext;
import com.github.dandelion.webanalytics.core.WebanalyticsConfiguration;
import com.github.dandelion.webanalytics.thymeleaf.dialect.DandelionWebAnalyticsDialect;
import com.github.dandelion.webanalytics.thymeleaf.dialect.WebAnalyticsAttributeName;
import org.thymeleaf.Arguments;
import org.thymeleaf.dom.Element;
import org.thymeleaf.processor.ProcessorResult;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * Processor for all web analytics attributes ({@link WebAnalyticsAttributeName})
 */
public class WebAnalyticsAttrProcessor extends DandelionAttrProcessor {

    public WebAnalyticsAttrProcessor(String attributeName) {
        super(attributeName);
    }

    @Override
    public int getPrecedence() {
        return DandelionWebAnalyticsDialect.HIGHEST_PRECEDENCE;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected ProcessorResult doProcessAttribute(Arguments arguments, Element element, String attributeName) {
        String strippedAttributeName
                = AttributesUtil.stripPrefix(attributeName, DandelionWebAnalyticsDialect.DIALECT_PREFIX);
        WebAnalyticsAttributeName webAnalyticsAttributeName
                = (WebAnalyticsAttributeName) AttributesUtil.find(strippedAttributeName, WebAnalyticsAttributeName.values());

        HttpServletRequest request = ArgumentsUtil.getWebContext(arguments).getHttpServletRequest();

        System.out.println(webAnalyticsAttributeName);
        if(webAnalyticsAttributeName == WebAnalyticsAttributeName.FINALIZER) {
            AssetsRequestContext context = AssetsRequestContext.get(request);
            AssetsFinalizerProcessorUtil.treat(context, arguments, request, element);
        } else {
            String value = element.getAttributeValue(attributeName);

            if (webAnalyticsAttributeName == WebAnalyticsAttributeName.TRACKER) {
                if("token".equals(value)) {
                    WebAnalyticsRequestContext.get(request)
                            .storeValue("provider", WebanalyticsConfiguration.getProvider())
                            .setup();
                } else if("all".equals(value)) {
                    WebAnalyticsRequestContext.get(request)
                            .storeValue("provider", WebanalyticsConfiguration.getProvider())
                            .storeValue("token", WebanalyticsConfiguration.getToken())
                            .setup();
                }
            } else {
                WebAnalyticsRequestContext.get(request)
                    .storeValue(webAnalyticsAttributeName.getAttribute(), value)
                    .setup();
            }

            AssetsFinalizerProcessorUtil.initialize(arguments, DandelionWebAnalyticsDialect.DIALECT_PREFIX);
        }
        return ProcessorResult.ok();
    }
}